import argparse
import sys

import glia
from glia.cli.start import StartCommand


def _optional_commands(parser):
    """
    Optional cli passed directly to the main parser.
    """
    parser.add_argument(
        "-V", "--version",
        version=f"%(prog)s {glia.__version__}",
        action="version"
    )
    parser.add_argument(
        "-v", "--verbose",
        help="increase output verbosity",
        action="store_true"
    )
    return parser


def _main_commands(parser):
    """
    Commands for the cli subparser is defined in
    :mod:`erin.cli` and called here.
    You can pass any of the arguments except `action` to these cli
    as defined by :func:`argparse.ArgumentParser.add_parser`.
    """
    StartCommand(
        parser,
        "start",
        aliases=["run"],
        help="start the gui"
    )
    return parser


def main(args=None):
    """The main entry point."""
    if args is None:
        args = sys.argv[1:]

    parser = argparse.ArgumentParser(
        description="Official Desktop Client for Glial Engines"
    )

    parser = _optional_commands(parser)
    parser_commands = parser.add_subparsers(
        title="Commands",
        dest="cli"
    )
    parser_commands = _main_commands(parser_commands)

    if isinstance(args, list):
        args = parser.parse_args(args)
    else:
        args = parser.parse_args()

    if args.verbose:

        # This is different from logging output verbosity. Enabling this
        # will print command internals directly to STDOUT regardless of
        # the settings defined in the logging module. Only recommended
        # for use by developers.
        print("Verbose Mode: Enabled")

    try:
        args.action(args)
    except AttributeError:
        pass

    if len(sys.argv[1:]) == 0:
        parser.print_help()
        parser.exit(0)


if __name__ == "__main__":
    main()



from PyQt5.QtWidgets import QTabWidget
from qtconsole.manager import QtKernelManager
from qtconsole.rich_jupyter_widget import RichJupyterWidget


class TabbedJupyterWidget(QTabWidget):
    def __init__(self, parent=None):
        super(TabbedJupyterWidget, self).__init__(parent)

        # Properties
        self.setTabPosition(QTabWidget.South)

        # Default Tab
        self.jupyter_widget = self.start_kernel('python')
        self.addTab(self.jupyter_widget, 'Python')

        # Create a tab creation method that also keeps track of the
        # jupyter objects created.

    def start_kernel(self, kernel: str):
        kernel_manager = QtKernelManager(kernel_name=kernel)
        kernel_manager.start_kernel()

        kernel_client = kernel_manager.client()
        kernel_client.start_channels()

        jupyter_widget = RichJupyterWidget()
        jupyter_widget.kernel_manager = kernel_manager
        jupyter_widget.kernel_client = kernel_client
        return jupyter_widget  # Binding this to a variable won't work.



import argparse

from glia import app
from glia.cli import CommandFactory


class StartCommand(CommandFactory):
    def __init__(self, parser, *args, **kwargs):
        self.parser = parser.add_parser(*args, **kwargs)
        self.parser.set_defaults(action=self.run)

        # Set Logging Levels
        self.choices = [
            'spam', 'debug', 'verbose', 'info', 'notice', 'warning',
            'success', 'error', 'critical'
        ]
        self.parser.add_argument(
            '--log',
            choices=self.choices,
            type=str.lower,
            help="amount of info to log"
        )

    def run(self, *sys_args, **kwargs):
        passed_args = {
            'log_level': None
        }
        if sys_args[0].log:
            passed_args['log_level'] = sys_args[0].log

        app.start(**passed_args)



import sys

from PyQt5 import Qsci
from PyQt5.Qsci import QsciScintilla
from PyQt5.QtGui import QFont

from glia.widgets.editor.lexers.python import PythonLexer


class Editor(Qsci.QsciScintilla):
    """
    Provides access to methods of editor.
    """
    def __init__(self, parent=None):
        """
        Loads default configuration for code_editor including
        the lexer depending on the kernel.
        """
        super(Editor, self).__init__(parent)

        # Fonts
        self.font = QFont()
        self.font.setFamily('Courier New')
        self.font.setFixedPitch(True)
        self.font.setPointSize(10)
        self.setFont(self.font)

        # Scrollbar
        self.SendScintilla(
            self.SCI_SETHSCROLLBAR,
            0
        )

        # Configurations
        self.setBraceMatching(QsciScintilla.SloppyBraceMatch)
        self.setIndentationsUseTabs(False)
        self.setIndentationGuides(True)
        self.setAutoIndent(True)
        self.setTabIndents(True)
        self.setUtf8(True)

        # Margin
        self.setMarginType(0, QsciScintilla.NumberMargin)
        self.setMarginWidth(0, "00")

        # Platform Specific
        if sys.platform.startswith("linux"):
            self.setEolMode(QsciScintilla.EolUnix)
        elif sys.platform.startswith("win32"):
            self.setEolMode(QsciScintilla.EolWindows)
        elif sys.platform.startswith("darwin"):
            self.setEolMode(QsciScintilla.EolMac)

        # Lexer
        self.lexer = PythonLexer(self, self.font)
        self.lexer.lock()

        # Slots
        self.linesChanged.connect(self.update_margin)

    def update_margin(self):
        """
        Adjust margin width to accommodate the number lines numbers.
        """
        lines = self.lines()
        self.setMarginWidth(0, "0" * (len(str(lines)) + 1))



from PyQt5.QtCore import QObject


class LexerBase(QObject):
    """
    Common properties of all lexers.
    """
    def __init__(self, code_editor, font):
        """
        :param code_editor: used to access the parent editor's methods
        :param font: settings in CodeEditor
        """
        self.code_editor = code_editor
        self.font = font



from PyQt5.QtCore import QObject, Qt
from PyQt5.QtWidgets import (QDockWidget, QListWidget, QMainWindow, QStatusBar,
                             QTabWidget, QWidget)

from glia.widgets.editor.tabs import EditorTabs
from glia.widgets.jupyter import TabbedJupyterWidget


class MainWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)

        # Menu Bar
        self.menu_bar = self.menuBar()
        self.file_menu = self.menu_bar.addMenu("File")
        self.edit_menu = self.menu_bar.addMenu("Edit")
        self.view_menu = self.menu_bar.addMenu("View")
        self.tools_menu = self.menu_bar.addMenu("Tools")

        # Project Browser
        self.project_dock = QDockWidget("Project", self)
        self.project_dock.setFloating(False)
        self.project_dock.setAllowedAreas(Qt.AllDockWidgetAreas)

        self.project = QListWidget(self)  # Mock Project Browser
        self.project.addItem("main.py")
        self.project.addItem("test.java")
        self.project.addItem("stats.r")
        self.project_dock.setWidget(self.project)
        self.addDockWidget(Qt.LeftDockWidgetArea, self.project_dock)

        # Workspace
        self.workspace = QTabWidget()
        self.workspace.setTabPosition(QTabWidget.South)
        self.setCentralWidget(self.workspace)

        # Editor
        self.editor_tabs = EditorTabs()
        self.workspace.addTab(self.editor_tabs, "Editor")

        # Monitor
        self.monitor = QWidget()
        self.workspace.addTab(self.monitor, "Monitor")

        # Jupyter Widget
        self.jupyter_dock = QDockWidget("Jupyter", self)
        self.jupyter_dock.setFloating(False)
        self.jupyter_dock.setAllowedAreas(Qt.AllDockWidgetAreas)

        self.jupyter = TabbedJupyterWidget()
        self.jupyter_dock.setWidget(self.jupyter)
        self.addDockWidget(Qt.RightDockWidgetArea, self.jupyter_dock)

        # Window Properties
        self.setWindowTitle("Mock Layout for Glia")

        # Status Bar
        self.status_bar = QStatusBar(self)
        self.setStatusBar(self.status_bar)

    def closeEvent(self, event):
        """
        Finish up any tasks. Stop all running widgets, it's threads and
        any I/O tasks.
        """
        for child in self.findChildren(QObject):
            if hasattr(child, "stop_running"):
                child.stop_running()



from abc import ABCMeta, abstractmethod


class CommandFactory(object):
    """
    Makes building commands for cli easier. Why not :class:`argparse.Action`?
    We tried doing that, but Action can't handle subparsers that have
    actions that need it's own extra sub cli. Plus this way, we
    have a neat structure for cli.
    When defining new sub commands, just pass your command's method
    name to the action key.
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def run(self, *args, **kwargs):
        """
        Kittens will die if this isn't implemented.
        """
        raise NotImplementedError(
            f"This method is not optional"
        )


if __name__ == "__main__":
    pass
